defmodule Nulla.Models.Activity do
  use Ecto.Schema
  import Ecto.Changeset
  alias Nulla.Repo
  alias Nulla.Snowflake

  @derive {Jason.Encoder, only: [:ap_id, :type, :actor, :object]}
  @primary_key {:id, :integer, autogenerate: false}
  schema "activities" do
    field :ap_id, :string
    field :type, :string
    field :actor, :string
    field :object, :string

    timestamps()
  end

  @doc false
  def changeset(activity, attrs) do
    activity
    |> cast(attrs, [:ap_id, :type, :actor, :object])
    |> validate_required([:ap_id, :type, :actor, :object])
    |> validate_inclusion(:type, ~w(Create Update Delete Undo Like Announce Follow Accept Reject))
  end

  def create_activity(attrs) do
    id = Map.get(attrs, :id, Snowflake.next_id())

    %__MODULE__{}
    |> __MODULE__.changeset(attrs)
    |> put_change(:id, id)
    |> Repo.insert()
  end
end
